//+------------------------------------------------------------------+
//|                                LuKu_MultiCurrencyStrength_v5.mq4 |
//|                                                             LuKu |
//|                                           http://www.luku.cz/fx/ |
//+------------------------------------------------------------------+
/*

  Indikator zobrazujici aktualni silu men
  Vychazi z MultiCurrencyIndex_en.mq5, CCFp, CC, JZI a pod.
  
  Zmeny:
  ------
    Verze 1.0 (13.2.2011)
      - zakladni naprogramovani
      
    Verze 2.0 (22.2.2011)
      - dodani RSI MA - vyhlazenejsi RSI
      
    Verze 3.0 (4.7.2011)
      - dodani komplexniho / presnejsiho vypoctu indexu ze vsech dostupnych men
      - zmena vychozich hodnot
      - dodani moznosti rozdilove krivky pro urcite meny
      - dodani automaticke korekce pri neexistenci pozadovaneho paru pro vypocet indexu meny
      - dodani cisilek pod barvami s poctem paru podilejicim se na vypoctu indexu, 7 je plny pocet, mene = nepresnost vypoctu
      
    Verze 4.0 (31.7.2011)
      - pridani krivek absolutnich hodnot men za MaxBarCount obdobi (vse za toto obdobi vmestna do spolecneho grafu), lze tak napr za 240baru zpet zobrazit silu/pomer men
      
    Verze 5.0 (10.12.2011)
      - zrychleno prekreslovani
      - odstraneno chybne prekreslovani diff krivky
      - dodana krivka ContractionExpansionCurve (kontrakce a expanze trhu) pro vsechny varianty
      - pridama varianta ABSMA pro hladsi prubeh absolutnich hodnot
      - dodana moznost exportu aktualnich hodnot do globalnich promenneych jako napr. MCS_EUR ...
  
*/
#property copyright "LuKu"
#property link      "http://www.luku.cz"
string Indicator_Name = "LuKu_MCS_v5";

#property indicator_separate_window
#property indicator_buffers 8

extern string sType = "--------- Typ indikatoru --------";
extern string IndType = "0,1,2,3,4,5 = RSIMA,RSI,MACD,Stoch,ABS,ABSMA";
extern int IndicatorType = 1;

extern string sDiff = "--------- Zobrazovat pouze rozdil paru --------";
extern bool ShowOnlyDiffCurve = false;
extern string DiffCurveFirst = "EUR";
extern string DiffCurveSecond = "USD";
extern string sCEC = "--------- Zobrazovat krivku expanze kontrakce --------";
extern bool   ContractionExpansionCurve = false;
extern double ContractionExpansionQntUp = 85;
extern double ContractionExpansionQntDown = 25;

extern string sSetting = "--------- Nastaveni promennych --------";
extern int RSI_period = 14; // period RSI
extern int RSI_MA     = 5;  // jak moc RSI vyhladit
extern int MACD_fast  = 9;  // period MACD_fast
extern int MACD_slow  = 26; // period MACD_slow
extern int Stoch_k    = 14; // period Stochastic %K
extern int Stoch_sma  = 5;  // period of smoothing for Stochastics %K
extern int ABS_MA     = 5;  // jak moc ABS vyhlazovat

extern string sColor = "--------- Barvy men --------";
extern color Color_USD = Green;
extern color Color_EUR = RoyalBlue;
extern color Color_GBP = Red;
extern color Color_CHF = Sienna;
extern color Color_JPY = Maroon;
extern color Color_AUD = DarkOrange;
extern color Color_CAD = Purple;
extern color Color_NZD = Teal;

extern string sOther = "--------- Ostatni --------";
extern double MaxBarCount = 1000; // pocet prepocitavanych baru 0 = vsechny, cislo omezeni
extern int Line_Thickness = 2;
extern int Line_width = 1;
extern int RefreshSec = 2; // obnovit kazdych N vterin
extern bool ExportToGlobal = false; // exportovat aktualni hodnoty do globalnich promennych (muze je tam vyuzivat jina strategie)

//---- buffers
// ceny paru vsechn 28 paru
double EURUSD[],
       EURGBP[],
       EURJPY[],
       EURCHF[],
       EURAUD[],
       EURCAD[],
       EURNZD[],
       GBPUSD[],
       USDJPY[],
       USDCHF[],
       NZDUSD[],
       AUDUSD[],
       USDCAD[],
       GBPJPY[],
       GBPCHF[],
       GBPNZD[],
       GBPAUD[],
       GBPCAD[],
       CHFJPY[],
       NZDJPY[],
       AUDJPY[],
       CADJPY[],
       NZDCHF[],
       AUDCHF[],
       CADCHF[],
       AUDNZD[],
       NZDCAD[],
       AUDCAD[];
        
// indexy men
double USDx[],
       EURx[],
       GBPx[],
       JPYx[],
       CHFx[],
       CADx[],
       AUDx[],
       NZDx[];
   
// pocet dostupnych symbolu    
int iUSD,
    iEUR,
    iGBP,
    iJPY,
    iCHF,
    iCAD,
    iAUD,
    iNZD;

// indexy men prumery pro RSI
// positive
double USDp[],
       EURp[],
       GBPp[],
       JPYp[],
       CHFp[],
       CADp[],
       AUDp[],
       NZDp[];
// negative
double USDn[],
       EURn[],
       GBPn[],
       JPYn[],
       CHFn[],
       CADn[],
       AUDn[],
       NZDn[];
       
// buffer pro vykreslovani                         
double USDplot[],
       EURplot[],
       GBPplot[],
       JPYplot[],
       CHFplot[],
       CADplot[],
       AUDplot[],
       NZDplot[];
// buffers of intermediate data schotastics by the close/close type without smoothing
double USDtemp[],
       EURtemp[],
       GBPtemp[],
       JPYtemp[],
       CHFtemp[],
       CADtemp[],
       AUDtemp[],
       NZDtemp[];

// ostatni promenne
int HWND = -1; // ID okna po initu
int i, ii; // for
int width = 1; // tloustka car
bool isInit = false; // pro zobrazovani textu
datetime newBarSec = 0; // zaznamenani casu noveho baru

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init(){

   Indicator_Name = StringConcatenate(Indicator_Name,"_",DoubleToStr(IndicatorType,0),"_",DoubleToStr(RSI_period*MACD_fast*MACD_slow*Stoch_k*Stoch_sma,0));
   if(ContractionExpansionCurve == true) Indicator_Name = StringConcatenate(Indicator_Name, "C");
   if(ShowOnlyDiffCurve == true) Indicator_Name = StringConcatenate(Indicator_Name, "D");
   IndicatorShortName(Indicator_Name);
   
   // nastaveni linek dle nastaveni
   if(ContractionExpansionCurve == false){
     SetLevelValue (0, 60);
     SetLevelValue (1, 40);
     SetLevelValue (2, 0);
     SetLevelValue (3, -60);
     SetLevelValue (4, -40);
   }

   // zobrazujeme vsechny krivky
   int wd = 0;
   if(0 > StringFind(Symbol(), "USD", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(0,DRAW_LINE, DRAW_LINE, wd, Color_USD);
   SetIndexBuffer(0,USDplot);
   SetIndexLabel(0, "USD");
   
   if(0 > StringFind(Symbol(), "EUR", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(1,DRAW_LINE, DRAW_LINE, wd, Color_EUR);
   SetIndexBuffer(1,EURplot);
   SetIndexLabel(1, "EUR");
  
   if(0 > StringFind(Symbol(), "GBP", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(2,DRAW_LINE, DRAW_LINE, wd, Color_GBP);
   SetIndexBuffer(2,GBPplot);
   SetIndexLabel(2, "GBP");
   
   if(0 > StringFind(Symbol(), "JPY", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(3,DRAW_LINE, DRAW_LINE, wd, Color_JPY);
   SetIndexBuffer(3,JPYplot);
   SetIndexLabel(3, "JPY");
   
   if(0 > StringFind(Symbol(), "CHF", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(4,DRAW_LINE, DRAW_LINE, wd, Color_CHF);
   SetIndexBuffer(4,CHFplot);
   SetIndexLabel(4, "CHF");
   
   if(0 > StringFind(Symbol(), "CAD", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(5,DRAW_LINE, DRAW_LINE, wd, Color_CAD);
   SetIndexBuffer(5,CADplot);
   SetIndexLabel(5, "CAD");
   
   if(0 > StringFind(Symbol(), "AUD", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(6,DRAW_LINE, DRAW_LINE, wd, Color_AUD);
   SetIndexBuffer(6,AUDplot);
   SetIndexLabel(6, "AUD");
   
   if(0 > StringFind(Symbol(), "NZD", 0)){wd = width;}else{wd = Line_Thickness;}
   SetIndexStyle(7,DRAW_LINE, DRAW_LINE, wd, Color_NZD);
   SetIndexBuffer(7,NZDplot);
   SetIndexLabel(7, "NZD");
   
   newBarSec = 0; // nulovani casu

   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit(){
   // umazani vytvorenych objektu
   for(int x=0;x<=18;x++) ObjectDelete(Indicator_Name +"_"+x);
   return(0);
}

// zobrazi texty
bool initText(){
   if(HWND < 0){
     HWND = WindowFind(Indicator_Name); // nelezeni okna indikatoru
     if(HWND < 0){
       Sleep(1000);
       return(0);
     }
   }
   if(isInit == true) return(true);
   if(ContractionExpansionCurve == true){
     // krivka expanze kontrakce
     TextCreate("Expansion / Contraction curve", 1, White, 4, 15);
     TextCreate("-", 2, Color_USD, 5, 0, 34);
   }else{
     if(ShowOnlyDiffCurve == true){
       TextCreate(StringConcatenate(DiffCurveFirst, "/", DiffCurveSecond),1, White, 4, 15);
       TextCreate("-", 2, Color_USD, 5, 0, 34);
     }else{
       TextCreate("USD EUR GBP JPY CHF CAD AUD NZD",1, White, 4, 15);
       TextCreate("-", 2, Color_USD, 5, 0, 34);
       TextCreate("-", 3, Color_EUR, 30, 0, 34);
       TextCreate("-", 4, Color_GBP, 55, 0, 34);
       TextCreate("-", 5, Color_JPY, 80, 0, 34);
       TextCreate("-", 6, Color_CHF, 105, 0, 34);
       TextCreate("-", 7, Color_CAD, 130, 0, 34);
       TextCreate("-", 8, Color_AUD, 155, 0, 34);
       TextCreate("-", 9, Color_NZD, 181, 0, 34);
     }
   }
   pairsCountTextCreate();
   TComment("Cekam na synchonizaci paru ...");
   isInit = true; // provedeme pouze jednou
}

// vypise pocet paru pomoci kterych se stanovil index meny
void pairsCountTextCreate(){
  int y = 32;
  int x = 7;
  int fontsize = 8;
  if(ShowOnlyDiffCurve == false && ContractionExpansionCurve == false){
    // zobraznovani cisel
    TextCreate(DoubleToStr(iUSD,0), 11, DarkGray, x+5, y, fontsize);
    TextCreate(DoubleToStr(iEUR,0), 12, DarkGray, x+30, y, fontsize);
    TextCreate(DoubleToStr(iGBP,0), 13, DarkGray, x+55, y, fontsize);
    TextCreate(DoubleToStr(iJPY,0), 14, DarkGray, x+80, y, fontsize);
    TextCreate(DoubleToStr(iCHF,0), 15, DarkGray, x+105, y, fontsize);
    TextCreate(DoubleToStr(iCAD,0), 16, DarkGray, x+130, y, fontsize);
    TextCreate(DoubleToStr(iAUD,0), 17, DarkGray, x+155, y, fontsize);
    TextCreate(DoubleToStr(iNZD,0), 18, DarkGray, x+181, y, fontsize);
  }else{
    // nezobrazovani cisel
    for(int j=11;j<=18;j++) ObjectDelete(Indicator_Name +"_"+j);
  }
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start(){
   // jsme stale pripojeni?
   if(!IsConnected()){
     return(0);
   }
   
   // pocitame jedou za x Seconds
   if(newBarSec > TimeCurrent()){
     return(0);
   }else{
     newBarSec = TimeCurrent()+RefreshSec;
   }
   
   // zobrazeni textu
   initText();

   // nezpracovana data
   int limit = IndicatorCounted();
   
   // je pozadovano pocitani pouze urciteho poctu baru (kvuli rychlosti)
   if(MaxBarCount > 0){
     if (limit > MaxBarCount)
       limit = MaxBarCount;
   }
   
   // RSI MA **************************************
   if(IndicatorType == 0){
     if(indexInit(limit + RSI_period + 1) == -1) return(0);
     // init temp pole
     ArrayResize(USDtemp, RSI_period+RSI_MA+limit+1);
     ArrayResize(EURtemp, RSI_period+RSI_MA+limit+1);
     ArrayResize(GBPtemp, RSI_period+RSI_MA+limit+1);
     ArrayResize(CHFtemp, RSI_period+RSI_MA+limit+1);
     ArrayResize(JPYtemp, RSI_period+RSI_MA+limit+1);
     ArrayResize(CADtemp, RSI_period+RSI_MA+limit+1);
     ArrayResize(AUDtemp, RSI_period+RSI_MA+limit+1);
     ArrayResize(NZDtemp, RSI_period+RSI_MA+limit+1);
     // naplnime indexy
     for(i = limit; i>=0; i--){
       USDtemp[i] = f_RSI(USDx,RSI_period,i, USDp, USDn);
       EURtemp[i] = f_RSI(EURx,RSI_period,i, EURp, EURn);
       GBPtemp[i] = f_RSI(GBPx,RSI_period,i, GBPp, GBPn);
       CHFtemp[i] = f_RSI(CHFx,RSI_period,i, CHFp, CHFn);
       JPYtemp[i] = f_RSI(JPYx,RSI_period,i, JPYp, JPYn);
       CADtemp[i] = f_RSI(CADx,RSI_period,i, CADp, CADn);
       AUDtemp[i] = f_RSI(AUDx,RSI_period,i, AUDp, AUDn);
       NZDtemp[i] = f_RSI(NZDx,RSI_period,i, NZDp, NZDn);
     }
     // a MA konverze
     for(i = limit-RSI_MA; i>=0; i--){
       USDplot[i] = SimpleMA(i,RSI_MA,USDtemp);
       EURplot[i] = SimpleMA(i,RSI_MA,EURtemp);
       GBPplot[i] = SimpleMA(i,RSI_MA,GBPtemp);
       CHFplot[i] = SimpleMA(i,RSI_MA,CHFtemp);
       JPYplot[i] = SimpleMA(i,RSI_MA,JPYtemp);
       CADplot[i] = SimpleMA(i,RSI_MA,CADtemp);
       AUDplot[i] = SimpleMA(i,RSI_MA,AUDtemp);
       NZDplot[i] = SimpleMA(i,RSI_MA,NZDtemp);
     }
     // mame pozadavek na jedinou krivku?
     plotDiffCurve(limit-RSI_MA);
   }
   // RSI ******************************************
   if(IndicatorType == 1){
     // naplnime indexy
     if(indexInit(limit + RSI_period + 1) == -1) return(0);
     for(i = limit; i>=0; i--){
       USDplot[i] = f_RSI(USDx,RSI_period,i, USDp, USDn);
       EURplot[i] = f_RSI(EURx,RSI_period,i, EURp, EURn);
       GBPplot[i] = f_RSI(GBPx,RSI_period,i, GBPp, GBPn);
       CHFplot[i] = f_RSI(CHFx,RSI_period,i, CHFp, CHFn);
       JPYplot[i] = f_RSI(JPYx,RSI_period,i, JPYp, JPYn);
       CADplot[i] = f_RSI(CADx,RSI_period,i, CADp, CADn);
       AUDplot[i] = f_RSI(AUDx,RSI_period,i, AUDp, AUDn);
       NZDplot[i] = f_RSI(NZDx,RSI_period,i, NZDp, NZDn);
     }
     // mame pozadavek na jedinou krivku?
     plotDiffCurve(limit);
   }
   // MACD *****************************************
   if(IndicatorType == 2){
     if(indexInit(limit + MathMax(MACD_fast, MACD_slow) + 1) == -1) return(0);
     for(i = limit; i>=0; i--){
       USDplot[i] = f_MACD(USDx,MACD_fast,MACD_slow,i);
       EURplot[i] = f_MACD(EURx,MACD_fast,MACD_slow,i);
       GBPplot[i] = f_MACD(GBPx,MACD_fast,MACD_slow,i);
       CHFplot[i] = f_MACD(CHFx,MACD_fast,MACD_slow,i);
       JPYplot[i] = f_MACD(JPYx,MACD_fast,MACD_slow,i);
       CADplot[i] = f_MACD(CADx,MACD_fast,MACD_slow,i);
       AUDplot[i] = f_MACD(AUDx,MACD_fast,MACD_slow,i);
       NZDplot[i] = f_MACD(NZDx,MACD_fast,MACD_slow,i);
     }
     // mame pozadavek na jedinou krivku?
     plotDiffCurve(limit);
   }
   // STOCHASTIC ***********************************
   if(IndicatorType == 3){
     if(indexInit(limit + MathMax(Stoch_k, Stoch_sma) + 1) == -1) return(0);
     // init Stoch pole
     ArrayResize(USDtemp, limit+1);
     ArrayResize(EURtemp, limit+1);
     ArrayResize(GBPtemp, limit+1);
     ArrayResize(CHFtemp, limit+1);
     ArrayResize(JPYtemp, limit+1);
     ArrayResize(CADtemp, limit+1);
     ArrayResize(AUDtemp, limit+1);
     ArrayResize(NZDtemp, limit+1);
     // predvypocet
     for(i = limit; i>=0; i--){
       USDtemp[i] = f_Stoch(USDx,Stoch_k,i);
       EURtemp[i] = f_Stoch(EURx,Stoch_k,i);
       GBPtemp[i] = f_Stoch(GBPx,Stoch_k,i);
       CHFtemp[i] = f_Stoch(CHFx,Stoch_k,i);
       JPYtemp[i] = f_Stoch(JPYx,Stoch_k,i);
       CADtemp[i] = f_Stoch(CADx,Stoch_k,i);
       AUDtemp[i] = f_Stoch(AUDx,Stoch_k,i);
       NZDtemp[i] = f_Stoch(NZDx,Stoch_k,i);
     }
     for(i = limit-Stoch_sma-1; i>=0; i--){
       USDplot[i] = SimpleMA(i,Stoch_sma,USDtemp);
       EURplot[i] = SimpleMA(i,Stoch_sma,EURtemp);
       GBPplot[i] = SimpleMA(i,Stoch_sma,GBPtemp);
       CHFplot[i] = SimpleMA(i,Stoch_sma,CHFtemp);
       JPYplot[i] = SimpleMA(i,Stoch_sma,JPYtemp);
       CADplot[i] = SimpleMA(i,Stoch_sma,CADtemp);
       AUDplot[i] = SimpleMA(i,Stoch_sma,AUDtemp);
       NZDplot[i] = SimpleMA(i,Stoch_sma,NZDtemp);
     }
     // mame pozadavek na jedinou krivku?
     plotDiffCurve(limit);
   }
   // ABS *****************************************
   if(IndicatorType == 4){
     if(indexInit(limit + 1) == -1) return(0);
     // zjistime stredni hodnotu indexu za nase obdobi a tu posadime na nulovou krivku, ziskame tak relativni stav meny vuci ostatnim
     double USDav = getAverageArrayValue(USDx);
     double EURav = getAverageArrayValue(EURx);
     double GBPav = getAverageArrayValue(GBPx);
     double CHFav = getAverageArrayValue(CHFx);
     double JPYav = getAverageArrayValue(JPYx);
     double CADav = getAverageArrayValue(CADx);
     double AUDav = getAverageArrayValue(AUDx);
     double NZDav = getAverageArrayValue(NZDx);
     // projeti a zobrazeni
     for(i = limit; i>=0; i--){
       USDplot[i] = USDx[i]-USDav;
       EURplot[i] = EURx[i]-EURav;
       GBPplot[i] = GBPx[i]-GBPav;
       CHFplot[i] = CHFx[i]-CHFav;
       JPYplot[i] = JPYx[i]-JPYav;
       CADplot[i] = CADx[i]-CADav;
       AUDplot[i] = AUDx[i]-AUDav;
       NZDplot[i] = NZDx[i]-NZDav;
     }
     // mame pozadavek na jedinou krivku?
     plotDiffCurve(limit);
   }
   // ABS MA *****************************************
   if(IndicatorType == 5){
     if(indexInit(limit + 1) == -1) return(0);
     // init temp pole
     ArrayResize(USDtemp, ABS_MA+limit+1);
     ArrayResize(EURtemp, ABS_MA+limit+1);
     ArrayResize(GBPtemp, ABS_MA+limit+1);
     ArrayResize(CHFtemp, ABS_MA+limit+1);
     ArrayResize(JPYtemp, ABS_MA+limit+1);
     ArrayResize(CADtemp, ABS_MA+limit+1);
     ArrayResize(AUDtemp, ABS_MA+limit+1);
     ArrayResize(NZDtemp, ABS_MA+limit+1);
     // zjistime stredni hodnotu indexu za nase obdobi a tu posadime na nulovou krivku, ziskame tak relativni stav meny vuci ostatnim
     double USDavx = getAverageArrayValue(USDx);
     double EURavx = getAverageArrayValue(EURx);
     double GBPavx = getAverageArrayValue(GBPx);
     double CHFavx = getAverageArrayValue(CHFx);
     double JPYavx = getAverageArrayValue(JPYx);
     double CADavx = getAverageArrayValue(CADx);
     double AUDavx = getAverageArrayValue(AUDx);
     double NZDavx = getAverageArrayValue(NZDx);
     // projeti a zobrazeni
     for(i = limit; i>=0; i--){
       USDtemp[i] = USDx[i]-USDavx;
       EURtemp[i] = EURx[i]-EURavx;
       GBPtemp[i] = GBPx[i]-GBPavx;
       CHFtemp[i] = CHFx[i]-CHFavx;
       JPYtemp[i] = JPYx[i]-JPYavx;
       CADtemp[i] = CADx[i]-CADavx;
       AUDtemp[i] = AUDx[i]-AUDavx;
       NZDtemp[i] = NZDx[i]-NZDavx;
     }
     // a MA konverze
     for(i = limit-ABS_MA; i>=0; i--){
       USDplot[i] = SimpleMA(i,ABS_MA,USDtemp);
       EURplot[i] = SimpleMA(i,ABS_MA,EURtemp);
       GBPplot[i] = SimpleMA(i,ABS_MA,GBPtemp);
       CHFplot[i] = SimpleMA(i,ABS_MA,CHFtemp);
       JPYplot[i] = SimpleMA(i,ABS_MA,JPYtemp);
       CADplot[i] = SimpleMA(i,ABS_MA,CADtemp);
       AUDplot[i] = SimpleMA(i,ABS_MA,AUDtemp);
       NZDplot[i] = SimpleMA(i,ABS_MA,NZDtemp);
     }
     // mame pozadavek na jedinou krivku?
     plotDiffCurve(limit-ABS_MA);
   }
   // export do globalnich promennych
   if(ExportToGlobal == true){
     GlobalVariableSet("MCS_USD", USDplot[0]);
     GlobalVariableSet("MCS_EUR", EURplot[0]);
     GlobalVariableSet("MCS_GBP", GBPplot[0]);
     GlobalVariableSet("MCS_CHF", CHFplot[0]);
     GlobalVariableSet("MCS_JPY", JPYplot[0]);
     GlobalVariableSet("MCS_CAD", CADplot[0]);
     GlobalVariableSet("MCS_AUD", AUDplot[0]);
     GlobalVariableSet("MCS_NZD", NZDplot[0]);
   }
   
   // aktualizace cisel paru indexu
   pairsCountTextCreate();
   return(0);
}
//+------------------------------------------------------------------+

// vrati prumernou (max-min)/2 hodnotu z dane sady hodnot
double getAverageArrayValue(double &aData[]){
  double min = aData[ArrayMinimum(aData)];
  double max = aData[ArrayMaximum(aData)];
  return(min+((max-min)/2));
}

void plotDiffCurve(int limit){
  // mame pozadavek na rozdilovou krivku?
  if(ContractionExpansionCurve == true){
    // zobrazovani expanze kontrakce
    double CEC = 0.0;
    double aData[];
    ArrayResize(aData, limit);
    int odecet = 0;
    switch(IndicatorType){
      case 0: // RSI
      case 1: // RSIMA
      case 3: // Stoch
        odecet = 50;
      case 2: // MACD
      case 4: // ABS
      case 5: // ABSMA
        for(i = limit; i>=0; i--){
          CEC = 0.0;
          CEC += MathAbs(USDplot[i] - odecet);
          CEC += MathAbs(EURplot[i] - odecet);
          CEC += MathAbs(GBPplot[i] - odecet);
          CEC += MathAbs(CHFplot[i] - odecet);
          CEC += MathAbs(JPYplot[i] - odecet);
          CEC += MathAbs(CADplot[i] - odecet);
          CEC += MathAbs(AUDplot[i] - odecet);
          CEC += MathAbs(NZDplot[i] - odecet);
          USDplot[i] = CEC;
          aData[i] = CEC;
          EURplot[i] = EMPTY_VALUE;
          GBPplot[i] = EMPTY_VALUE;
          CHFplot[i] = EMPTY_VALUE;
          JPYplot[i] = EMPTY_VALUE;
          CADplot[i] = EMPTY_VALUE;
          AUDplot[i] = EMPTY_VALUE;
          NZDplot[i] = EMPTY_VALUE;
        }
        break;
    }
    // upravime linky
    SetLevelValue (0, calculateQuantile(aData, 50)); // median
    SetLevelValue (1, calculateQuantile(aData, ContractionExpansionQntUp));
    SetLevelValue (2, calculateQuantile(aData, ContractionExpansionQntDown));
    
  }else{
    if(ShowOnlyDiffCurve == true){
      //for(i = limit-RSI_MA; i>=0; i--){
      for(i = limit; i>=0; i--){
        // a ostatni jsou prazdne
        USDplot[i] = getPlotValue(DiffCurveFirst, i) - getPlotValue(DiffCurveSecond, i);
        EURplot[i] = EMPTY_VALUE;
        GBPplot[i] = EMPTY_VALUE;
        CHFplot[i] = EMPTY_VALUE;
        JPYplot[i] = EMPTY_VALUE;
        CADplot[i] = EMPTY_VALUE;
        AUDplot[i] = EMPTY_VALUE;
        NZDplot[i] = EMPTY_VALUE;
      }
    }
  }
}
// ziska pozadovanou hodnotu z Plot promennych
double getPlotValue(string DCF, int pos){
  if(DCF == "USD") return(USDplot[pos]);
  if(DCF == "EUR") return(EURplot[pos]);
  if(DCF == "GBP") return(GBPplot[pos]);
  if(DCF == "CHF") return(CHFplot[pos]);
  if(DCF == "JPY") return(JPYplot[pos]);
  if(DCF == "CAD") return(CADplot[pos]);
  if(DCF == "AUD") return(AUDplot[pos]);
  if(DCF == "NZD") return(NZDplot[pos]);
  return(0);
}

// vypisuje textove hlaseni do okna indikatoru
void TextCreate(string txt, int ObjId = 0, color col = White, int x = 0, int y = 0, int FontSize = 7){
   if(HWND < 0) return(0);
   string ID = Indicator_Name +"_"+ObjId;
   if(ObjectFind(ID) != 1){
     ObjectCreate(ID, OBJ_LABEL, HWND, 0, 0);
     ObjectSet(ID, OBJPROP_XDISTANCE, x);
     ObjectSet(ID, OBJPROP_YDISTANCE, y);
   }
   ObjectSetText(ID, txt, FontSize, "Verdana", col);
}

// komentar na poslednim radku
void TComment(string txt, color Barva = Crimson){
  TextCreate(txt, 10, Barva, 4, 48, 8);
}

// provede skopirovani uzaviracich cen daneho symbolu do pole
int CopyClose(string symbol, int Count, double &Destination[], bool reverse){
  // pokud symbol neexistuje nulujeme promennou
  if(iClose(symbol,0,0) == 0){
    // symbol neexistuje
    ArrayResize(Destination, 0);
    return(1);
  }
  // zmenime velikosti
  ArrayResize(Destination, Count+1);
  double cPrice = 0;
  for(int i=0; i<=Count; i++){
    cPrice = converPrice(symbol,i,reverse);
    if(cPrice == 0) return(-1);
    Destination[i] = cPrice;
  }
  return(1);
}

// konvertuje cenu na spravne hodnoty pro vypocet
double converPrice(string symbol, int position, bool reverse){
  double price;
  price = MarketInfo(symbol, MODE_POINT) * MarketInfo(symbol, MODE_LOTSIZE);
  if(price == 0) return(0);
  price = iClose(symbol,0,position) / price;
  if(price == 0) return(0);
  if(reverse == true){
    price = 1 / price; // pro sell je nutno otocit pomer
  }
  return(price);
}

//+------------------------------------------------------------------+
///                        Calculating RSI
//+------------------------------------------------------------------+
double f_RSI(double &buf_in[], int period, int shift, double &buf_pos[], double &buf_neg[]){
   double pos = 0.0, neg = 0.0, diff = 0.0;
   if(buf_pos[shift+1] == 0 && buf_neg[shift+1] == 0){
     // spocitame prumerny zisk, poprve
     for(int j=shift; j <= shift+period; j++){
       diff = buf_in[j] - buf_in[j+1];
       if(diff > 0){
         pos += diff;
       }else{
         neg -= diff;
       }
     }
     pos /= period;
     neg /= period;
     // tuto hodnotu zaznamename
     buf_pos[shift] = pos;
     buf_neg[shift] = neg;
   }else{
     // ziskame onu hodnotu
     diff = buf_in[shift] - buf_in[shift+1];
     if(diff > 0){
       pos = diff;
     }else{
       neg = -diff;
     }
     buf_pos[shift] = ((buf_pos[shift+1]*(period-1))+pos) / period;
     buf_neg[shift] = ((buf_neg[shift+1]*(period-1))+neg) / period;
   }
   // vraceni hodnoty
   if(buf_neg[shift] == 0) return(0.0);
   return(100.0 -(100.0/(1.0+(buf_pos[shift] / buf_neg[shift]))));
}

//+------------------------------------------------------------------+
///                        Calculating MACD
//+------------------------------------------------------------------+   
double f_MACD(double &buf_in[],int period_fast,int period_slow,int shift){
   return(-1 * (1/SimpleMA(shift,period_fast,buf_in) - 1/SimpleMA(shift,period_slow,buf_in)));
}

//+------------------------------------------------------------------+
///                        Calculating SMA
//+------------------------------------------------------------------+   
double SimpleMA(int position, int period, double &price[]){
   double result=0.0;
   for(int i=0;i<period;i++) result+=price[position+i];
   result/=period;
   return(result);
}

//+------------------------------------------------------------------+
///        Calculating Stochastic close/close without smoothing
//+------------------------------------------------------------------+   
double f_Stoch(double &price[],int period_k,int shift){
   double result=0.0;
   double max=price[ArrayMaximum(price,period_k, shift)];
   double min=price[ArrayMinimum(price,period_k, shift)];
   result=(price[shift]-min)/(max-min)*100.0;
   return(result);
}

// naplni promenne pozadovanymi indexy
int indexInit(int bufferLimit){
   // ziskani cen paru
   // nahrani history dat, pokud se vypisuje Cekam na ... je nutne dohrat rucne data - otevrit pak v MT4 a pomoci PageUp nacist history. hlaska sama zmizi
   if(CopyClose("EURUSD", bufferLimit, EURUSD, false) == -1){TComment("Cekam na history data EURUSD ..."); return(-1);}
   if(CopyClose("EURGBP", bufferLimit, EURGBP, false) == -1){TComment("Cekam na history data EURGBP ..."); return(-1);}
   if(CopyClose("EURJPY", bufferLimit, EURJPY, false) == -1){TComment("Cekam na history data EURJPY ..."); return(-1);}
   if(CopyClose("EURCHF", bufferLimit, EURCHF, false) == -1){TComment("Cekam na history data EURCHF ..."); return(-1);}
   if(CopyClose("EURAUD", bufferLimit, EURAUD, false) == -1){TComment("Cekam na history data EURAUD ..."); return(-1);}
   if(CopyClose("EURCAD", bufferLimit, EURCAD, false) == -1){TComment("Cekam na history data EURCAD ..."); return(-1);}
   if(CopyClose("EURNZD", bufferLimit, EURNZD, false) == -1){TComment("Cekam na history data EURNZD ..."); return(-1);}
   if(CopyClose("GBPUSD", bufferLimit, GBPUSD, false) == -1){TComment("Cekam na history data GBPUSD ..."); return(-1);}
   if(CopyClose("USDJPY", bufferLimit, USDJPY, false) == -1){TComment("Cekam na history data USDJPY ..."); return(-1);}
   if(CopyClose("USDCHF", bufferLimit, USDCHF, false) == -1){TComment("Cekam na history data USDCHF ..."); return(-1);}
   if(CopyClose("NZDUSD", bufferLimit, NZDUSD, false) == -1){TComment("Cekam na history data NZDUSD ..."); return(-1);}
   if(CopyClose("AUDUSD", bufferLimit, AUDUSD, false) == -1){TComment("Cekam na history data AUDUSD ..."); return(-1);}
   if(CopyClose("USDCAD", bufferLimit, USDCAD, false) == -1){TComment("Cekam na history data USDCAD ..."); return(-1);}
   if(CopyClose("GBPJPY", bufferLimit, GBPJPY, false) == -1){TComment("Cekam na history data GBPJPY ..."); return(-1);}
   if(CopyClose("GBPCHF", bufferLimit, GBPCHF, false) == -1){TComment("Cekam na history data GBPCHF ..."); return(-1);}
   if(CopyClose("GBPNZD", bufferLimit, GBPNZD, false) == -1){TComment("Cekam na history data GBPNZD ..."); return(-1);}
   if(CopyClose("GBPAUD", bufferLimit, GBPAUD, false) == -1){TComment("Cekam na history data GBPAUD ..."); return(-1);}
   if(CopyClose("GBPCAD", bufferLimit, GBPCAD, false) == -1){TComment("Cekam na history data GBPCAD ..."); return(-1);}
   if(CopyClose("CHFJPY", bufferLimit, CHFJPY, false) == -1){TComment("Cekam na history data CHFJPY ..."); return(-1);}
   if(CopyClose("NZDJPY", bufferLimit, NZDJPY, false) == -1){TComment("Cekam na history data NZDJPY ..."); return(-1);}
   if(CopyClose("AUDJPY", bufferLimit, AUDJPY, false) == -1){TComment("Cekam na history data AUDJPY ..."); return(-1);}
   if(CopyClose("CADJPY", bufferLimit, CADJPY, false) == -1){TComment("Cekam na history data CADJPY ..."); return(-1);}
   if(CopyClose("NZDCHF", bufferLimit, NZDCHF, false) == -1){TComment("Cekam na history data NZDCHF ..."); return(-1);}
   if(CopyClose("AUDCHF", bufferLimit, AUDCHF, false) == -1){TComment("Cekam na history data AUDCHF ..."); return(-1);}
   if(CopyClose("CADCHF", bufferLimit, CADCHF, false) == -1){TComment("Cekam na history data CADCHF ..."); return(-1);}
   if(CopyClose("AUDNZD", bufferLimit, AUDNZD, false) == -1){TComment("Cekam na history data AUDNZD ..."); return(-1);}
   if(CopyClose("NZDCAD", bufferLimit, NZDCAD, false) == -1){TComment("Cekam na history data NZDCAD ..."); return(-1);}
   if(CopyClose("AUDCAD", bufferLimit, AUDCAD, false) == -1){TComment("Cekam na history data AUDCAD ..."); return(-1);}
   TComment(" "); // nulovani synchronizacni hlasky
   
   // uprava pole na pozadovanou velikost
   ArrayResize(USDx, bufferLimit+1);
   ArrayResize(EURx, bufferLimit+1);
   ArrayResize(GBPx, bufferLimit+1);
   ArrayResize(CHFx, bufferLimit+1);
   ArrayResize(JPYx, bufferLimit+1);
   ArrayResize(CADx, bufferLimit+1);
   ArrayResize(AUDx, bufferLimit+1);
   ArrayResize(NZDx, bufferLimit+1);
   // rsi buffery
   // negative
   ArrayResize(USDn, bufferLimit+1);
   ArrayResize(EURn, bufferLimit+1);
   ArrayResize(GBPn, bufferLimit+1);
   ArrayResize(CHFn, bufferLimit+1);
   ArrayResize(JPYn, bufferLimit+1);
   ArrayResize(CADn, bufferLimit+1);
   ArrayResize(AUDn, bufferLimit+1);
   ArrayResize(NZDn, bufferLimit+1);
   // positive
   ArrayResize(USDp, bufferLimit+1);
   ArrayResize(EURp, bufferLimit+1);
   ArrayResize(GBPp, bufferLimit+1);
   ArrayResize(CHFp, bufferLimit+1);
   ArrayResize(JPYp, bufferLimit+1);
   ArrayResize(CADp, bufferLimit+1);
   ArrayResize(AUDp, bufferLimit+1);
   ArrayResize(NZDp, bufferLimit+1);

   // projedeme bary a zjistime silu men
   for(i = bufferLimit; i>=0; i--){
      // vypocet sily men
      USDx[i] = getUsdIndex(i);
      EURx[i] = getEurIndex(i);
      GBPx[i] = getGbpIndex(i);
      CHFx[i] = getChfIndex(i);
      JPYx[i] = getJpyIndex(i);
      CADx[i] = getCadIndex(i);
      AUDx[i] = getAudIndex(i);
      NZDx[i] = getNzdIndex(i);
   }
}

// ziskani USD indexu ze vsech men
double getUsdIndex(int pos){
  iUSD = 0;
  double value = 0.0;
  value += getPairsValue(EURUSD, pos, iUSD, true);
  value += getPairsValue(GBPUSD, pos, iUSD, true);
  value += getPairsValue(USDJPY, pos, iUSD, false);
  value += getPairsValue(USDCHF, pos, iUSD, false);
  value += getPairsValue(NZDUSD, pos, iUSD, true);
  value += getPairsValue(AUDUSD, pos, iUSD, true);
  value += getPairsValue(USDCAD, pos, iUSD, false);
  return(value/iUSD);
}

// ziskani EUR indexu ze vsech men
double getEurIndex(int pos){
  iEUR = 0;
  double value = 0.0;
  value += getPairsValue(EURUSD, pos, iEUR, false);
  value += getPairsValue(EURGBP, pos, iEUR, false);
  value += getPairsValue(EURJPY, pos, iEUR, false);
  value += getPairsValue(EURCHF, pos, iEUR, false);
  value += getPairsValue(EURAUD, pos, iEUR, false);
  value += getPairsValue(EURCAD, pos, iEUR, false);
  value += getPairsValue(EURNZD, pos, iEUR, false);
  return(value/iEUR);
}

// ziskani GBP indexu ze vsech men
double getGbpIndex(int pos){
  iGBP = 0;
  double value = 0.0;
  value += getPairsValue(EURGBP, pos, iGBP, true);
  value += getPairsValue(GBPUSD, pos, iGBP, false);
  value += getPairsValue(GBPJPY, pos, iGBP, false);
  value += getPairsValue(GBPCHF, pos, iGBP, false);
  value += getPairsValue(GBPNZD, pos, iGBP, false);
  value += getPairsValue(GBPAUD, pos, iGBP, false);
  value += getPairsValue(GBPCAD, pos, iGBP, false);
  return(value/iGBP);
}

// ziskani JPY indexu ze vsech men
double getJpyIndex(int pos){
  iJPY = 0;
  double value = 0.0;
  value += getPairsValue(EURJPY, pos, iJPY, true);
  value += getPairsValue(USDJPY, pos, iJPY, true);
  value += getPairsValue(GBPJPY, pos, iJPY, true);
  value += getPairsValue(CHFJPY, pos, iJPY, true);
  value += getPairsValue(NZDJPY, pos, iJPY, true);
  value += getPairsValue(AUDJPY, pos, iJPY, true);
  value += getPairsValue(CADJPY, pos, iJPY, true);
  return(value/iJPY);
}

// ziskani CHF indexu ze vsech men
double getChfIndex(int pos){
  iCHF = 0;
  double value = 0.0;
  value += getPairsValue(EURCHF, pos, iCHF, true);
  value += getPairsValue(USDCHF, pos, iCHF, true);
  value += getPairsValue(GBPCHF, pos, iCHF, true);
  value += getPairsValue(CHFJPY, pos, iCHF, false);
  value += getPairsValue(NZDCHF, pos, iCHF, true);
  value += getPairsValue(AUDCHF, pos, iCHF, true);
  value += getPairsValue(CADCHF, pos, iCHF, true);
  return(value/iCHF);
}

// ziskani NZD indexu ze vsech men
double getNzdIndex(int pos){
  iNZD = 0;
  double value = 0.0;
  value += getPairsValue(EURNZD, pos, iNZD, true);
  value += getPairsValue(NZDUSD, pos, iNZD, false);
  value += getPairsValue(GBPNZD, pos, iNZD, true);
  value += getPairsValue(NZDJPY, pos, iNZD, false);
  value += getPairsValue(NZDCHF, pos, iNZD, false);
  value += getPairsValue(AUDNZD, pos, iNZD, true);
  value += getPairsValue(NZDCAD, pos, iNZD, false);
  return(value/iNZD);
}

// ziskani AUD indexu ze vsech men
double getAudIndex(int pos){
  iAUD = 0;
  double value = 0.0;
  value += getPairsValue(EURAUD, pos, iAUD, true);
  value += getPairsValue(AUDUSD, pos, iAUD, false);
  value += getPairsValue(GBPAUD, pos, iAUD, true);
  value += getPairsValue(AUDJPY, pos, iAUD, false);
  value += getPairsValue(AUDCHF, pos, iAUD, false);
  value += getPairsValue(AUDNZD, pos, iAUD, false);
  value += getPairsValue(AUDCAD, pos, iAUD, false);
  return(value/iAUD);
}

// ziskani CAD indexu ze vsech men
double getCadIndex(int pos){
  iCAD = 0;
  double value = 0.0;
  value += getPairsValue(EURCAD, pos, iCAD, true);
  value += getPairsValue(USDCAD, pos, iCAD, true);
  value += getPairsValue(GBPCAD, pos, iCAD, true);
  value += getPairsValue(CADJPY, pos, iCAD, false);
  value += getPairsValue(CADCHF, pos, iCAD, false);
  value += getPairsValue(NZDCAD, pos, iCAD, true);
  value += getPairsValue(AUDCAD, pos, iCAD, true);
  return(value/iCAD);
}

// vrati hodnotu paru a nebo 0 + inkrementuje inc pro delitel
double getPairsValue(double &pairs[], int pos, int &inc, bool revers = false){
  if(ArraySize(pairs) == 0){
    // par neexistuje
    return(0);
  }else{
    // inkrementujeme delitel
    inc++;
    // vracime jeho hodnotu
    if(revers == true){
      return(1/pairs[pos]);
    }else{
      return(pairs[pos]);
    }
  }
}

// z luku knihovny
// vypocet hodnoty kvantilu dle vstupnich dat
// aData - double hodnoty v poli
// perceptil - hodnota v procentech (25 1.kvartil, 50 median/2. kvartil, 75 3.kvartil)
double calculateQuantile(double aData[], double perceptil = 50){
  double qKoef = perceptil * (ArraySize(aData) + 1) / 100.0; // koeficient, napr. 8.25
  // serazeni dat dle hodnot
  ArraySort(aData);
  // pokud je qKoef vetsi jak max hodnota vracime max hodnotu
  if(qKoef > ArraySize(aData)){
    return(aData[ArraySize(aData)-1]);
  }
  if(qKoef < 1){
    return(aData[0]);
  }
  // ziskani min a max hodnoty
  int qMin = MathFloor(qKoef); // 8
  int qMax = MathCeil(qKoef);  // 9
  // vytazeni vlastnich hodnot
  double dMin = aData[qMin-1]; // 10
  double dMax = aData[qMax-1]; // 20
  // linearni interpolace TODO lze interpolovat nekolika zpusoby
  double qValue = dMin + ((dMax - dMin) * (qKoef - qMin)); // 10 + ((20-10) * (8.25 - 8)) => 10 + (10*.25) = 12.5
  return(qValue);
}